package util.sec;

import java.security.Principal;
import java.util.Map;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.FailedLoginException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;

import util.HibernateUtil;

import entity.sec.User;

public class GPackLoginModule implements LoginModule{
  private Subject subject;
  private CallbackHandler callbackHandler;
  
  private boolean succeeded = false;
  
  private boolean commitSucceeded = false;
  
  private String username;
  
  private char[] password;
  
  private User user;
  
  private Role role;
  
  @Override
  public boolean abort() throws LoginException{
    if(succeeded == false) return false;
    else if(succeeded == true && commitSucceeded == false){
      succeeded = false;
      username = null;
      if(password != null){
        for(int i = 0; i < password.length; i++) password[i] = ' ';
        password = null;
      }
      user = null;
    }else logout();
    
    return true;
  }

  @Override
  public boolean commit() throws LoginException{
    if(!succeeded) return false;
    
    // add a Principal (authenticated identity)
    // to the Subject
    
    if(!subject.getPrincipals().contains(user))
      subject.getPrincipals().add(user);
    if(!subject.getPrincipals().contains(role))
      subject.getPrincipals().add(role);

    // in any case, clean out state
    username = null;
    for(int i = 0; i < password.length; i++) password[i] = ' ';
    password = null;

    commitSucceeded = true;
      
    return true;
  }
  
  @SuppressWarnings("rawtypes")
	@Override
  public void initialize(Subject subject, CallbackHandler callbackHandler,
                         Map sharedState, Map options){
    this.subject = subject;
    this.callbackHandler = callbackHandler;
  }

  @Override
  public boolean login() throws LoginException{
    if(callbackHandler == null)
      throw new LoginException("Erro: no h CallbackHandler disponvel");

    Callback[] callbacks = new Callback[2];
    callbacks[0] = new NameCallback("user name: ");
    callbacks[1] = new PasswordCallback("password: ", false);
       
    try{
      callbackHandler.handle(callbacks);
      username = ((NameCallback)callbacks[0]).getName();
      
      char[] tmpPassword = ((PasswordCallback)callbacks[1]).getPassword();
      if(tmpPassword == null) tmpPassword = new char[0];
      password = new char[tmpPassword.length];
      System.arraycopy(tmpPassword, 0, password, 0, tmpPassword.length);
      ((PasswordCallback)callbacks[1]).clearPassword();
   
    }catch(java.io.IOException e){
      throw new LoginException(e.toString());
    }catch(UnsupportedCallbackException e){
      throw new LoginException("Error: " + e.getCallback().toString() +
        " does not return user credentials");
    }

    Principal[] principals = null;
    principals = HibernateUtil.validateUser(username, password);
    if(principals != null && principals.length == 2){
      user = (User)principals[0];
      role = (Role)principals[1];      
      succeeded = true;
      
      return true;
    }
     
    // authentication failed -- clean out state
    succeeded = false;
    username = null;
    for (int i = 0; i < password.length; i++) password[i] = ' ';
    password = null;
    
    throw new FailedLoginException("User name/password wrong");
  }

  @Override
  public boolean logout() throws LoginException{
    subject.getPrincipals().remove(user);
    succeeded = false;
    succeeded = commitSucceeded;
    username = null;
    if(password != null){
      for(int i = 0; i < password.length; i++) password[i] = ' ';
      password = null;
    }
    user = null;
    
    return true;
  }

}
